/*********************************
DAQ SYSTEM
 EASIROC & CAMAC ver1.4
readout
 32CH MPPCs ADC + 16CH PMT ADC
made 2014.08.06 by H.ITO
---------
コンパイル
---------
 $cd DAQ
 $make clean
 $make
---------
実行コマンド
---------
	$./daq 192.168.10.16 3
	ctrl + Shift + T >>new tab of tarminal
	$cd UDPcontrol
	$./udp  ... HV調節プログラム
**********************************/
#include"src/easiroc.h"
#include"src/SC.hh"

#include<fstream>
#include<ios>

#include<stdio.h>
#include<stdlib.h>
#include<iostream>
#include<fstream>
#include<sys/stat.h>
//#include "/usr/local/camlib/camlib.c"
//#include "/usr/local/camlib/camlib.h"

static const int  SiTCP_PORT       = 24;

extern unsigned int slowdata[sizeByte_SC];
extern unsigned int ReadSCdata[sizeByte_RSC];
extern unsigned int Probedata[sizeByte_PSC];
extern unsigned int Pindata[sizeByte_PIN];

int ForceStop = 0;
int EndADC = 0;

void ForceStop_FromCtrl_C(int signal);
int  receive(int sock, char* data_buf, unsigned int *ReadLength);
int  WriteData(int socket, unsigned int data);
int  ReadData(int socket, unsigned int signal, unsigned int data);
void ADCStop(int socket);

int DebugFPGA(int socket);
int TransmitSC(int socket);
int TransmitReadSC(int socket);
int TransmitProbe(int socket);

int ContinuousADC_ver5(int socket);
//int ContinuousADC_ver4(int socket);
int ADCOneCycle_wHeader_ver5(int socket, std::ofstream& file);
//int ADCOneCycle_wHeader_ver4(int socket, std::ofstream& file);

void MHTDC_debug(int socket);
int Debugging(int socket);
int chipselect();

using namespace std;


//*********************
//     main
//*********************
int main(int argc, char* argv[]){
  if(1 == argc){
    std::cout << "Usage\n";
    std::cout << "./easiroc [IP Address of SOY] [DAQ mode]" << std::endl;
    std::cout << "   DAQ mode : 1 MHTDC\n";
    std::cout << "            : 2 ADC\n";
    std::cout << "            : 3 ADC & MHTDC" << std::endl;
    exit(-1);
  }

  unsigned int daq_mode = (unsigned int)atoi(argv[2]);

  const char* SiTCP_MASTER_IP = argv[1];

  int sock = 0;
  struct sockaddr_in SiTCP_ADDR;
  const char*  IP;
  unsigned int port;

  //easiroc Initialize -------------------------------------------------------------
  ForceStop  = 0;
 (void) signal(SIGINT, ForceStop_FromCtrl_C);

  sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
  IP                         = SiTCP_MASTER_IP;
  port                       = SiTCP_PORT;
  SiTCP_ADDR.sin_family      = AF_INET;
  SiTCP_ADDR.sin_port        = htons(port);
  SiTCP_ADDR.sin_addr.s_addr = inet_addr(IP);

  cout<<"sock:"<<sock<<endl;
  cout<<"IP:"<<IP<<endl;
  cout<<"port:"<<port<<endl;
  cout<<"AF_NET:"<<AF_INET<<endl;

  struct timeval tv;
  tv.tv_sec  = 3;
  tv.tv_usec = 0;
  setsockopt(sock, SOL_SOCKET, SO_RCVTIMEO, (char*)&tv, sizeof(tv));

  int flag = 1;
  setsockopt(sock, IPPROTO_TCP, TCP_NODELAY, (char*)&flag, sizeof(flag));

//CAMAC initialize-----------------------------------------
//	COPEN();
//	CGENZ();


 //Connection -------------------------------------------------------------
//  if(0 > connect(sock, (struct sockaddr*)&SiTCP_ADDR, sizeof(SiTCP_ADDR))){
  if(0 > bind(sock, (struct sockaddr*)&SiTCP_ADDR, sizeof(SiTCP_ADDR))){
    std::cerr << "SiTCP Master :: Connection fail" << std::endl;
    close(sock);
    return -1;
  }
//  cout<<"debug"<<endl;
  
  std::cout << "SiTCP Master :: Connection Done" << std::endl;
  std::cout << std::endl;

  // Auto Initialize -------------------------------------------------------
//  PrepareFPGA();
//  DebugFPGA(sock);
  std::cout << "ASIS Initialize : Done" << std::endl;
  std::cout << std::endl;
  usleep(10000);

  WriteData(3, 37888);
  PrepareSC(1);
  TransmitSC(sock);
  std::cout << "Slow Control chip1 : Done" << std::endl;
  std::cout << std::endl;
  PrepareReadSC(1);
  TransmitReadSC(sock);
  std::cout << "Read Slow Control chip1 : Done" << std::endl;
  std::cout << std::endl;

  WriteData(3, 21504);
  PrepareSC(2);
  TransmitSC(sock);
  std::cout << "Slow Control chip2 : Done" << std::endl;
  std::cout << std::endl;
  PrepareReadSC(2);
  TransmitReadSC(sock);
  std::cout << "Read Slow Control chip2: Done" << std::endl;
  std::cout << std::endl;

  WriteData(3, 5120);
  {
    unsigned int signal = 31;
    signal = signal << 16;
    unsigned int data = daq_mode;
    data = data << 8;
    signal += data;
    if(-1 == WriteData(sock, signal)){
      exit(-1);
    }
    std::cout << "#D : DAQ mode is " << daq_mode << std::endl;
  }

//DAQ mode start
ContinuousADC_ver5(sock);


//End Connection ---------------------------------------------------------
close(sock);
std::cout << "All :: Connection Close" << std::endl;

return 0;}


void ForceStop_FromCtrl_C(int signal){
  std::cout << "Stop request from user" << std::endl;
  ForceStop = 1;
  EndADC = 1;
}

int receive(int sock, char* data_buf, unsigned int *ReadLength){
  unsigned int revd_size     = 0;
  int tmp_returnVal = 0;

  while(revd_size < *ReadLength){
    tmp_returnVal = recv(sock, data_buf +revd_size, *ReadLength -revd_size, 0);
    if(tmp_returnVal < 0){
      int errbuf = errno;
      perror("TCP receive");
      if(errbuf == EAGAIN){
	if(EndADC == 1){
	  std::cout << "No data" << std::endl << std::flush;
	  break;
	}
	continue;
      }else if(errbuf == EINTR){
	continue;
      }else{
	exit(-1);
      }
    }
    revd_size += tmp_returnVal;
  }

    //    std::cout << "ret is " << std::dec << revd_size << std::endl;
  return revd_size;
}

int WriteData(int sock, unsigned int data){
  data += 128 << 24;
  send(sock, (char*)&data, sizeof(int), 0);
  unsigned int buf = 0;
  unsigned int length = 4;
  receive(sock, (char*)&buf, &length);
  if(data - buf){
    std::cerr << "### Data Transmit Error ###" << std::endl;
    std::cerr << "Transmited data is " << std::hex << data << std::endl;
    std::cout << "Returned data is   " << std::hex << buf << std::endl;
    return -1;
  }

  return 0;
}

int ReadData(int sock, unsigned int signal, unsigned int *data){
  signal += 64 << 24;
  send(sock, (char*)&signal, sizeof(int), 0);
  unsigned int length = 4;
  receive(sock, (char*)data, &length);
  return 0;
}

int DebugFPGA(int socket){
  unsigned int buffer = 0;
  buffer += 0 << 16;
  buffer += (Pindata[0] & 255) << 8;
  if(-1 == WriteData(socket, buffer)){
    exit(-1);
  }

  for(int i = 1 ; i<5; ++i){
    buffer = 0;
    if(i == 4){
      buffer += 5 << 16;      
    }else{
      buffer += i << 16;
    }
    buffer += ((Pindata[1] >> (i-1)*8) & 255) << 8;

    if(-1 == WriteData(socket, buffer)){
      exit(-1);
    }
    usleep(1);
  }
  
  return 0;
}

int TransmitSC(int socket){
  unsigned int data = 0;
  //Set SC mode -----------------------------------------------------------
  data = 0;
  data += 1 << 16;
  data += 240 << 8;
  if(-1 == WriteData(socket, data)){
    exit(-1);
  }

  //SC start -------------------------------------------------------------
  data = 0;
  data += 10 << 16;
  data += (slowdata[0] & 255) << 8;
  if(-1 == WriteData(socket, data)){
    exit(-1);
  }

  for(int i = 1; i<15; ++i){
    for(int shift = 0; shift<4; ++shift){
      data = 0;
      data += 10 << 16;
      data += ((slowdata[i] >> 8*shift) & 255) << 8;
      if(-1 == WriteData(socket, data)){
	exit(-1);
      }
      //std::cout<<"test"<<std::endl;
      usleep(1);
    }
  }


  //StartCycle -----------------------------------------------------------
  data = 0;
  data += 1 << 16;
  data += 242 << 8;
  if(-1 == WriteData(socket, data)){
    exit(-1);
  }

  data = 0;
  data += 1 << 16;
  data += 240 << 8;
  if(-1 == WriteData(socket, data)){
    exit(-1);
  }


  //Load SC --------------------------------------------------------------
  data = 0;
  data += 1 << 16;
  data += 241 << 8;
  if(-1 == WriteData(socket, data)){
    exit(-1);
  }

  data = 0;
  data += 1 << 16;
  data += 240 << 8;
  if(-1 == WriteData(socket, data)){
    exit(-1);
  }

  return 0;
}

int TransmitReadSC(int socket){

  //SCA read ---------------------------------------------------------------
  unsigned int data = 0;

  for(int i = 0; i<4; ++i){
    data = 0;
    data += 12 << 16;
    data += ((ReadSCdata[0] >> i*8) & 255) << 8;
    if(-1 == WriteData(socket, data)){
      exit(-1);
    }
    usleep(1);
  }

  //StartCycle ------------------------------------------------------------
  data = 0;
  data += 1 << 16;
  data += 242 << 8;
  if(-1 == WriteData(socket, data)){
    exit(-1);
  }

  data = 0;
  data += 1 << 16;
  data += 240 << 8;
  if(-1 == WriteData(socket, data)){
    exit(-1);
  }
  
  return 0;
}

int TransmitProbe(int socket){
  unsigned int data = 0;
  //Set Probe mode --------------------------------------------------------
  data += 1 << 16;
  data += 208 << 8;
  if(-1 == WriteData(socket, data)){
    exit(-1);
  }

  //Probe start ----------------------------------------------------------
  for(int i = 0; i<sizeByte_PSC; ++i){
    for(int shift = 0; shift<4; ++shift){
      data = 0;
      data += 10 << 16;
      data += ((Probedata[i] >> 8*shift) & 255) << 8;
      if(-1 == WriteData(socket, data)){
	exit(-1);
      }
      usleep(1);
    }
  }

  usleep(50000);

  //StartCycle ------------------------------------------------------------
  data = 0;
  data += 1 << 16;
  data += 210 << 8;
  if(-1 == WriteData(socket, data)){
    exit(-1);
  }

  data = 0;
  data += 1 << 16;
  data += 208 << 8;
  if(-1 == WriteData(socket, data)){
    exit(-1);
  }

  return 0;
}


//**********************
//  トリガーモード v5
int ContinuousADC_ver5(int socket){
	char name[500];//,cmd[100];
	string dirname, a,filename;
	int RunNo;

	ifstream ifsa("./setup/SaveFileName.txt");
	ifsa>>dirname;//アドレス先/home/hiroshi/exp/exp40/data
	ifsa.close();

	ifstream ifsb("./setup/WorkDIR.txt");
	ifsb>>name;//アドレス先/home/hiroshi/exp/exp40/
	ifsb.close();

	sprintf(name,"%srunnum.txt",name);
	ifstream ifsc(name);
	ifsc>>name;
	RunNo=atoi(name);
	ifsc.close();

	//上書き確認
	sprintf(name,"%srun%05d.esd",dirname.c_str(),RunNo);
	cout << "生データ転送先: "<<name<<"\n" << flush;
	struct stat st1;
	int ret1 = stat(name,&st1);
	if(0 != ret1){}
	else{
		cout<<"message::"<<"データを上書きしますか？ Yes[1]/No[0]>>";
		cin >>a;
/*	int i_loop=0;
  for(i_loop=0;i_loop<2;i_loop++){
   if(a!="0"&&a!="1"){
    cout<<"message::"<<"もう一度入力してください Yes[1]/No[0]]>>";
    cin>>a;
    if(a=="1")break;
    else if(a=="0"||i_loop==1){
			cout<<"3回以上間違いました。\nDAQモード終了\n";
			return 0;
     }}}*/

		if(a=="0"){
			cout<<"DAQモード終了\n";
			return 0;
		//if(a==0)exit(1);
		}}

  int MaxCycleNum = 0, m=0;
// cout << "最大イベント数を入力してください(int型)：" <<flush;
//  cin >> MaxCycleNum;
	ifstream ifs_d("./setup/EventNum.txt");
	ifs_d>>name;
	MaxCycleNum=atoi(name);
 cout << "最大イベント数：" <<MaxCycleNum<<endl<<flush;

  a="q";
	int i_loop=0;
  for(i_loop=0;i_loop<2;i_loop++){
  if(a!="0"&&a!="1"){
    cout<<"message::"<<"DAQ start Yes[1]/No[0]]>>";
    cin>>a;
    if(a=="1")break;
    else if(a=="0"||i_loop==1){
			cout<<"DAQモード終了\n";
			return 0;
  }}}

//	sprintf(name,"%s/%s",dirname.c_str(),file_name.c_str());
	sprintf(name,"%srun%05d.esd",dirname.c_str(),RunNo);
	ofstream file(name,ios::binary);
//	ofstream ofs ("../Rawdata/ttmp");

  unsigned int signal = 0;
  int EventNum = 0;

  signal += 32 << 24;
//  signal += 64 << 24;
  signal += 100 << 16;
  send(socket, (char*)&signal, sizeof(int), 0);

 while(EventNum < MaxCycleNum){
  ADCOneCycle_wHeader_ver5(socket, file);
  EventNum++;
  if(0 == EventNum%1){
   cout <<"."<< flush;
   m++;
   if(0 == m%10)cout <<"\n"<< flush;
  }
  if(false||EventNum  == MaxCycleNum||ForceStop == 1){
   ADCStop(socket);
   EndADC = 1;
   int abort = 0;
   while(0 == ADCOneCycle_wHeader_ver5(socket, file)){
    usleep(1000);
    if(abort == 50){
	   ADCStop(socket);
	   abort = 0;
	   }
   cout << "."<<flush;
   ++abort;
      }
      EndADC    = 0;
      ForceStop = 0;
      break;
    }
  }
  cout << "End ADC" << endl;
  file.close();
  return 0;
}


//**********************
//  トリガーモード v4
/*int ContinuousADC_ver4(int socket){
	ifstream ifs("./setup/SaveFileName.txt");
	string dirname;
	ifs>>dirname;

  cout << "生データ転送先: "<<dirname<<"\n" << flush;
  string file_name;
  cout << "ファイル名を書いてください: " << flush;
  cin >> file_name;

	//上書き確認
	char name[500];
	string a;
	sprintf(name,"%s/%s",dirname.c_str(),file_name.c_str());
	struct stat st1;
	int ret1 = stat(name,&st1);
	if(0 != ret1){}
	else{
		cout<<"message::"<<"データを上書きしますか？ Yes[1]/No[0]>>";
		cin >>a;
	int i_loop=0;
  for(i_loop=0;i_loop<2;i_loop++){
   if(a!="0"&&a!="1"){
    cout<<"message::"<<"もう一度入力してください Yes[1]/No[0]]>>";
    cin>>a;
    if(a=="1")break;
    else if(a=="0"||i_loop==1){
			cout<<"3回以上間違いました。\nDAQモード終了\n";
			return 0;
     }}}

		if(a=="0"){
			cout<<"DAQモード終了\n";
			return 0;
		//if(a==0)exit(1);
		}}

  int MaxCycleNum = 0, m=0;
  cout << "最大イベント数を入力してください(int型)：" <<flush;
  cin >> MaxCycleNum;
  a="q";
	int i_loop=0;
  for(i_loop=0;i_loop<2;i_loop++){
   if(a!="0"&&a!="1"){
    cout<<"message::"<<"DAQ start Yes[1]/No[0]]>>";
    cin>>a;
    if(a=="1")break;
    else if(a=="0"||i_loop==1){
			cout<<"DAQモード終了\n";
			return 0;
  }}}

	sprintf(name,"%s/%s",dirname.c_str(),file_name.c_str());
	ofstream file(name,ios::binary);
//	ofstream ofs ("../Rawdata/ttmp");

  unsigned int signal = 0;
  int EventNum = 0;

  signal += 32 << 24;
  signal += 100 << 16;
  send(socket, (char*)&signal, sizeof(int), 0);

 while(EventNum < MaxCycleNum){
  ADCOneCycle_wHeader_ver4(socket, file);
  EventNum++;
  if(0 == EventNum%1){
   cout <<"."<< flush;
   m++;
   if(0 == m%10)cout <<"\n"<< flush;
  }
  if(false||EventNum  == MaxCycleNum||ForceStop == 1){
   ADCStop(socket);
   EndADC = 1;
   int abort = 0;
   while(0 == ADCOneCycle_wHeader_ver4(socket, file)){
    usleep(10000);
    if(abort == 50){
	   ADCStop(socket);
	   abort = 0;
	   }
   cout << "."<<flush;
   ++abort;
      }
      EndADC    = 0;
      ForceStop = 0;
      break;
    }
  }
  cout << "End ADC" << endl;
  file.close();
  return 0;
}*/



//**********************
//  データ収集v5
int ADCOneCycle_wHeader_ver5(int socket, std::ofstream& file){
	//determined
  static const int NofHeader = 3;

  unsigned int DataBuffer_ve3[1000];
  memset(DataBuffer_ve3, 0, sizeof(DataBuffer_ve3));
  unsigned int *ptr_DataBuf = DataBuffer_ve3;
  unsigned int TotalRecvByte  = 0;
  unsigned int TotalRecvEvent = 0;

	//CAMAC determined
//	const int MOD_NUM1 = 5;//ADC port
//	const int MOD_NUM2 = 2;//TDC port
//	const int SUB_NUM1[16] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
//	int data1,q1,x1;
//	int data_camac[16];

  static unsigned int sizeHeader = NofHeader*sizeof(int);

  unsigned int Header[NofHeader] = {1, 1, 1};
  int ret = 0;
  ret = receive(socket, (char*)Header, &sizeHeader);
  if(ret <= 0 && EndADC == 1){
    return -1;
  }
//************
//ヘッダー
//************
  if(Header[0] != 0xFFFFEA0C){
    std::cerr << "Fatal ADC ERROR : HEADER" << std::endl;
    std::cerr << "Header1 : " << std::hex << Header[0] 
	      << " Header2 : " << std::hex << Header[1]
	      << " Header3 : " << std::hex << Header[2] << std::endl;
    exit(-1);
  }
	else{
    unsigned int sizeData = (sizeof(int)*Header[1] & 0xffff);
				//total MPCs data size
    unsigned int NofWord = (Header[1] & 0xffff);
    unsigned int OneData[NofWord];
    receive(socket, (char*)OneData, &sizeData);

    memcpy(ptr_DataBuf, Header, sizeHeader);
    ptr_DataBuf += NofHeader;

    memcpy(ptr_DataBuf, OneData, sizeData);
    ptr_DataBuf += sizeData/sizeof(int);

//************
//CAMAC ADC
//************
//		unsigned int sizeData_CAMAC = (sizeof(int)*0x1& 0xffff);
//		CAMAC(NAF(MOD_NUM1,SUB_NUM1[0],8),&data1,&q1,&x1);//ADC event
//		if(q1==1){
//			for(int i_loop=0;i_loop<16;i_loop++){
//				//CAMAC ADC Reading
//				CAMAC(NAF(MOD_NUM1,SUB_NUM1[i_loop],0),&data1,&q1,&x1);
//				data1+=0x71000000;
//				data1+=0x10000*i_loop;
//				memcpy(ptr_DataBuf, (char*)&data1, sizeData_CAMAC);
//				ptr_DataBuf += sizeData_CAMAC/sizeof(int);
//			}
				//CAMAC TDC Reading
//				int trr=1;
//				for(int i_loop=0;i_loop<16;i_loop++){
//					CAMAC(NAF(MOD_NUM2,SUB_NUM1[i_loop],8),&data1,&q1,&x1);
//					trr*=q1;}
//				if(trr==1){
//				for(int i_loop=0;i_loop<16;i_loop++){
//				CAMAC(NAF(MOD_NUM2,SUB_NUM1[i_loop],0),&data1,&q1,&x1);
//				data1+=0x62000000;
//				data1+=0x10000*i_loop;
//				memcpy(ptr_DataBuf, (char*)&data1, sizeData_CAMAC);
//				ptr_DataBuf += sizeData_CAMAC/sizeof(int);
//				}}
//		 }


    TotalRecvByte += sizeHeader + sizeData;//HEADER+MPPC用のメモリ
//		for(int i_loop=0;i_loop<32;i_loop++)
//		for(int i_loop=0;i_loop<32;i_loop++)
//     TotalRecvByte += sizeData_CAMAC;//ADC用のメモリ
    // TotalRecvByte += sizeData_CAMAC;//TDC用のメモリ
//    TotalRecvEvent++;
//		for(int i_loop=0;i_loop<16;i_loop++)
//	    CAMAC(NAF(MOD_NUM1,SUB_NUM1[i_loop],9),&data1,&q1,&x1);
  }

  if(EndADC != 1){
    for(unsigned int i = 0; i<TotalRecvByte/sizeof(int); i++){
      unsigned int buffer = DataBuffer_ve3[i];
      file.write((char*)&buffer, sizeof(int));
   }}
  return 0;
}


/*int ADCOneCycle_wHeader_ver4(int socket, std::ofstream& file){
	//determined
  static const int NofHeader = 3;

  unsigned int DataBuffer_ve3[1000];
  memset(DataBuffer_ve3, 0, sizeof(DataBuffer_ve3));
  unsigned int *ptr_DataBuf = DataBuffer_ve3;
  unsigned int TotalRecvByte  = 0;
  unsigned int TotalRecvEvent = 0;

	//CAMAC determined
	const int MOD_NUM1 = 5;//ADC port
	const int MOD_NUM2 = 2;//TDC port
	const int SUB_NUM1[16] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
	int data1,q1,x1;
//	int data_camac[16];

  static unsigned int sizeHeader = NofHeader*sizeof(int);

  unsigned int Header[NofHeader] = {1, 1, 1};
  int ret = 0;
  ret = receive(socket, (char*)Header, &sizeHeader);
  if(ret <= 0 && EndADC == 1){
    return -1;
  }
//************
//ヘッダー
//************
  if(Header[0] != 0xFFFFEA0C){
    std::cerr << "Fatal ADC ERROR : HEADER" << std::endl;
    std::cerr << "Header1 : " << std::hex << Header[0] 
	      << " Header2 : " << std::hex << Header[1]
	      << " Header3 : " << std::hex << Header[2] << std::endl;
    exit(-1);
  }
	else{
    unsigned int sizeData = (sizeof(int)*Header[1] & 0xffff);
				//total MPCs data size
    unsigned int NofWord = (Header[1] & 0xffff);
    unsigned int OneData[NofWord];
    receive(socket, (char*)OneData, &sizeData);

    memcpy(ptr_DataBuf, Header, sizeHeader);
    ptr_DataBuf += NofHeader;

    memcpy(ptr_DataBuf, OneData, sizeData);
    ptr_DataBuf += sizeData/sizeof(int);

//************
//CAMAC ADC
//************
		unsigned int sizeData_CAMAC = (sizeof(int)*0x1& 0xffff);
		CAMAC(NAF(MOD_NUM1,SUB_NUM1[0],8),&data1,&q1,&x1);//ADC event
		if(q1==1){
			for(int i_loop=0;i_loop<16;i_loop++){
				//CAMAC ADC Reading
				CAMAC(NAF(MOD_NUM1,SUB_NUM1[i_loop],0),&data1,&q1,&x1);
				data1+=0x71000000;
				data1+=0x10000*i_loop;
				memcpy(ptr_DataBuf, (char*)&data1, sizeData_CAMAC);
				ptr_DataBuf += sizeData_CAMAC/sizeof(int);
			}
				//CAMAC TDC Reading
				int trr=1;
				for(int i_loop=0;i_loop<16;i_loop++){
					CAMAC(NAF(MOD_NUM2,SUB_NUM1[i_loop],8),&data1,&q1,&x1);
					trr*=q1;}
				if(trr==1){
				for(int i_loop=0;i_loop<16;i_loop++){
				CAMAC(NAF(MOD_NUM2,SUB_NUM1[i_loop],0),&data1,&q1,&x1);
				data1+=0x62000000;
				data1+=0x10000*i_loop;
				memcpy(ptr_DataBuf, (char*)&data1, sizeData_CAMAC);
				ptr_DataBuf += sizeData_CAMAC/sizeof(int);
				}}
		 }


    TotalRecvByte += sizeHeader + sizeData;//HEADER+MPPC用のメモリ
		for(int i_loop=0;i_loop<32;i_loop++)
     TotalRecvByte += sizeData_CAMAC;//ADC用のメモリ
    // TotalRecvByte += sizeData_CAMAC;//TDC用のメモリ
    TotalRecvEvent++;
		for(int i_loop=0;i_loop<16;i_loop++)
	    CAMAC(NAF(MOD_NUM1,SUB_NUM1[i_loop],9),&data1,&q1,&x1);
  }

  if(EndADC != 1){
    for(unsigned int i = 0; i<TotalRecvByte/sizeof(int); i++){
      unsigned int buffer = DataBuffer_ve3[i];
      file.write((char*)&buffer, sizeof(int));
   }}
  return 0;
}*/


// Debug ---------------------------------------------------------------------
int Debugging(int socket){
  enum debug{read, write, quit, sizedebug};
  bool flag_[sizedebug];
  for(int i = 0; i<sizedebug; ++i){
    flag_[i] = false;
  }

  std::cout << "Read/Write" << std::endl;
  std::cout << "1. Read" << std::endl;
  std::cout << "2. Write" << std::endl;
  std::cout << "3. Return" << std::endl;
  std::cout << "input # =====> " << std::flush;
  char buf[10];
  fgets(buf, sizeof(buf), stdin);
  char num;
  sscanf(buf, "%c", &num);
  
  switch(num){
  case '1':
    flag_[read] = true;
    break;
    
  case '2':
    flag_[write] = true;
    break;
    
  default:
    
    break;
  };

  if(flag_[read]){
    std::cout << "Debug Read" << std::endl;
    std::cout << "Input subaddress : " << std::flush;
    std::string strbuf;
    std::cin >> strbuf;
    std::istringstream valbuf(strbuf);
    unsigned int saddr = 0;
    valbuf >> saddr;
    saddr = saddr << 16;
    
    unsigned int data = 0;
    if(0 == ReadData(socket, saddr, &data)){
      std::cout << std::hex << data << std::endl;
      std::cout << "bit31 <<                    >> bit0" << std::endl;
      for(int i = 1; i<33; ++i){
	if(data >> (32 -i) & 1){
	  std::cout << "!" << std::flush;
	}else{
	  std::cout << "." << std::flush;
	}
	
	if(0 == i%8){
	  std::cout <<  " " << std::endl;
	}
      }
      std::cout << std::endl;
    }else{
      std::cerr << "Read error" << std::endl;
      exit(-1);
    }
  }

  if(flag_[write]){
    std::cout << "Debug Write" << std::endl;
    std::cout << "Input subaddress : " << std::flush;
    std::string strbuf;
    std::cin >> strbuf;
    std::istringstream valbuf(strbuf);
    unsigned int signal = 0;
    valbuf >> signal;
    signal = signal << 16;
    
    std::cout << "Input signal : " << std::flush;
    std::string strbuf2;
    std::cin >> strbuf2;
    std::istringstream databuf(strbuf2);
    unsigned int data = 0;
    databuf >> data;
    data = data << 8;
    signal += data;
    
    std::cout << std::hex << signal << std::flush;
    std::cout << std::dec << std::endl;
    std::cout << signal << std::endl;
    if(-1 == WriteData(socket, signal)){
      exit(-1);
    }
  }

  return 0;
}

void ADCStop(int socket){
  unsigned int signal = 0;
  std::cout << "ADC exit process" << std::endl;
  signal += 16 << 24;
  signal += 100 << 16;
  send(socket, (char*)&signal, sizeof(int), 0);
  sleep(1);
  
  signal = 0;
  signal += 100 << 16;
  send(socket, (char*)&signal, sizeof(int), 0);

  usleep(10000);

  return;
}

void MHTDC_debug(int sock){

  std::ofstream m_file("mhtdc.dat",std::ios::binary);

  unsigned int saddr = 50;
  unsigned int data = 0;
  saddr = saddr << 16;
  unsigned int signal = 4;
  signal = signal << 8;
  saddr += signal;
  WriteData(sock, saddr);

  saddr = 50;
  saddr = saddr << 16;
  WriteData(sock, saddr);
  usleep(100000);

  while(ForceStop == 0){
    //  for(int k = 0; k<3; ++k){
    saddr = 41;
    data = 0;
    saddr = saddr << 16;

    usleep(100000);
    if(0 == ReadData(sock, saddr, &data)){
      int size = (data >> 8) & 0xff;
      //      std::cout << "size : " << size << std::endl;
      std::cout << std::hex << "status : " << data << std::endl;
      
      int valid = data & 0x1;

      if(valid == 1){
	unsigned int f_size = 0xffff;
	f_size = (f_size << 16) + size;
	m_file.write((char*)&f_size, sizeof(unsigned int));
	for(int i = 0; i<size; ++i){
	  //	  usleep(100);
	  saddr = 42;
	  saddr = saddr << 16;
	  unsigned int head =0;
	  if(0 == ReadData(sock, saddr, &data)){
	    std::cout << std::hex << "header" << i << " : " << data << std::endl;
	    head = data & 0xffff;
	  }else{
	    std::cerr << "Read error" << std::endl;
	    exit(-1);
	  }

	  // Leading 
	  saddr = 40;
	  saddr = saddr << 16;
	  if(0 == ReadData(sock, saddr, &data)){
	    unsigned int time = (data & 0xffff);
	    unsigned int one_data = (head << 16) + time;
	    m_file.write((char*)&one_data, sizeof(unsigned int));
	    std::cout << std::dec << "time L" << i << " : " << time << " ns" << std::endl;
	  }else{
	    std::cerr << "Read error" << std::endl;
	    exit(-1);
	  }

	  // Trailing
	  saddr = 43;
	  saddr = saddr << 16;
	  if(0 == ReadData(sock, saddr, &data)){
	    unsigned int time = (data & 0xffff);
	    time = (1 << 15) + time;
	    unsigned int one_data = (head << 16) + time;
	    m_file.write((char*)&one_data, sizeof(unsigned int));
	    std::cout << std::dec << "time T" << i << " : " << (time & 0xfff) << " ns" << std::endl;
	  }else{
	    std::cerr << "Read error" << std::endl;
	    exit(-1);
	  }
	}
	saddr = 50;
	saddr = saddr << 16;
	signal = 4;
	signal = signal << 8;
	saddr += signal;
	WriteData(sock, saddr);

	saddr = 50;
	saddr = saddr << 16;
	WriteData(sock, saddr);

      }
    }else{
      std::cerr << "Read error" << std::endl;
      exit(-1);
    }
  }

  return;
}

int chipselect(){

  WriteData(3, 37888);
    // if(-1 == WriteData(3, 5120)){
    //   exit(-1);
    // }
  return 0;
}
